<?php
/**
 * Get order list.
 * 
 * @param  string    $mode 
 * @param  mix       $value 
 * @param  string    $orderBy 
 * @param  object    $pager 
 * @access public
 * @return array
 */
public function getList($mode, $value, $orderBy = 'id_desc', $pager = null)
{
    $days = $this->config->shop->confirmLimit;

    if($days)
    {
        $deliveryDate = date('Y-m-d H:i:s', time() - 24 * 60 * 60 * $days);
        $this->dao->update(TABLE_ORDER)
            ->set('deliveryStatus')->eq('confirmed')
            ->where('deliveryStatus')->eq('send')
            ->andWhere('deliveriedDate')->le($deliveryDate)
            ->exec();
    }

    $orders = $this->dao->select('t1.*')->from(TABLE_ORDER)->alias('t1')
        ->leftJoin(TABLE_ADDRESS)->alias('t2')->on('t1.account = t2.account')
        ->beginIf($mode == 'account')->where('t1.account')->eq($value)->fi()
        ->beginIf($mode == 'status')->where('t1.status')->eq($value)->fi()
        ->beginIf(!commonModel::isAvailable('score'))->andWhere('t1.type')->ne('score')->fi()
        ->beginIf(!commonModel::isAvailable('shop'))->andWhere('t1.type')->ne('shop')->fi()
        ->beginIf($this->get->contact)->andwhere('t2.contact')->eq($this->get->contact)->fi()
        ->beginIf($this->get->phone)->andwhere('t2.phone')->eq($this->get->phone)->fi()
        ->orderBy($orderBy)
        ->page($pager)
        ->fetchAll('id');

    $products = $this->dao->select('*')->from(TABLE_ORDER_PRODUCT)
        ->where('orderID')->in(array_keys($orders))
        ->beginIf($this->get->product)->andWhere('productID')->eq($this->get->product)->fi()
        ->fetchGroup('orderID');

    foreach($orders as $key => $order)
    {
        if($this->get->product and !isset($products[$order->id]))
        {
            unset($orders[$key]);
            continue;
        }
        $order->products = isset($products[$order->id]) ? $products[$order->id] : array();
    }

    return $orders;
}

/**
 * Get product infomation posted to buy.
 * 
 * @param  string $product 
 * @param  int    $count 
 * @access public
 * @return void
 */
public function getPostedProducts($product, $count = 0)
{
    $productIdList = is_array($product) ? $product : array($product);

    /* Get products(use groupBy to distinct products).  */
    $products = $this->dao->select('t1.*, t2.category')->from(TABLE_PRODUCT)->alias('t1')
        ->leftJoin(TABLE_RELATION)->alias('t2')->on('t1.id = t2.id')
        ->where('t1.id')->in($productIdList)
        ->andWhere('t2.type')->eq('product')
        ->beginIF(RUN_MODE == 'front')->andWhere('t1.status')->eq('normal')->fi()
        ->fetchAll('id');

    if(empty($products)) return array();

    /* Get categories for these products. */
    $categories = $this->dao->select('t2.id, t2.name, t2.alias, t1.id AS product')
        ->from(TABLE_RELATION)->alias('t1')
        ->leftJoin(TABLE_CATEGORY)->alias('t2')->on('t1.category = t2.id')
        ->where('t2.type')->eq('product')
        ->andWhere('t1.id')->in(array_keys($products))
        ->fetchGroup('product', 'id');

    /* Assign categories to it's product. */
    foreach($products as $product) $product->categories = !empty($categories[$product->id]) ? $categories[$product->id] : array();

    /* Get images for these products. */
    $images = $this->loadModel('file')->getByObject('product', array_keys($products), $isImage = true);

    /* Assign images to it's product. */
    foreach($products as $product)
    {
        if(empty($images[$product->id])) continue;
        $product->image = new stdclass();
        if(isset($images[$product->id]))  $product->image->list = $images[$product->id];
        if(!empty($product->image->list)) $product->image->primary = $product->image->list[0];

    }

    $goodsList = array();
    foreach($productIdList as $id => $product)
    {
        $goods = new stdclass();

        $product = zget($products, $product);

        $goods->product    = $product->id;
        $goods->name       = $product->name;
        $goods->image      = isset($product->image) ? $product->image : '';
        $goods->alias      = $product->alias;
        $goods->category   = $product->category;
        $goods->categories = $product->categories;
        $goods->count      = $this->post->count[$id];
        $goods->amount     = $product->amount;
        $goods->extra      = (object) $this->post->extra[$id];
        $goods->attributes = $this->loadModel('attribute')->getPairs(array_keys($product->categories));

        $goods->price = $this->loadModel('product')->getPriceByAttr($product, $goods->extra);
        $goods->promotion = 0;

        $goodsList[$id] = $goods;
    }

    return $goodsList;
}

/**
 * Create an order.
 * 
 * @access public
 * @return void
 */
public function create()
{
    $order = fixer::input('post')
        ->add('account', $this->app->user->account)
        ->add('createdDate', helper::now())
        ->add('payStatus', 'not_paid')
        ->add('status', 'normal')
        ->add('deliveryStatus', 'not_send')
        ->add('type', 'shop')
        ->get();

    $address = $this->dao->select('*')->from(TABLE_ADDRESS)->where('id')->eq($this->post->deliveryAddress)->andWhere('account')->eq($this->app->user->account)->fetch();
    $order->address = helper::jsonEncode($address);

    if($this->post->createAddress)
    {
        $address = $this->createAddress();
        if(!$address) return array('result' => 'fail', 'message' => dao::getError());
        $order->address = helper::jsonEncode($address);
    }

    $this->dao->insert(TABLE_ORDER)
        ->data($order, 'createAddress,deliveryAddress,zipcode,phone,contact,price,count,product,extra')
        ->autocheck()
        ->batchCheck($this->config->order->require->create, 'notempty')
        ->exec();
    if(dao::isError()) return array('result' => 'fail', 'message' => dao::getError());

    $orderID = $this->dao->lastInsertID();

    $goods = new stdclass();
    $goods->orderID = $orderID;

    if(!$this->post->product)
    {
        $this->clearOrder($orderID);
        return array('result' => 'fail', 'message' => $this->lang->order->noProducts);
    }
    
    /* Save products of the order and compute order amount. */
    $amount = 0;
    foreach($this->post->product as $id => $product)
    {
        $product = $this->dao->select('*')->from(TABLE_PRODUCT)->where('id')->eq($product)->fetch();
        if(empty($product)) continue;

        $goods->productID   = $product->id; 
        $goods->productName = $product->name; 
        $goods->count       = $this->post->count[$id];
        $goods->extra       = json_encode($this->post->extra[$id]);

        if(isset($this->config->product->stock) && $this->config->product->stock)
        {
            if($product->amount < $goods->count)
            {
                $this->clearOrder($orderID);
                return array('result' => 'fail', 'message' => sprintf($this->lang->order->lowStocks, $goods->productName));
            }
        }

        $goods->price = $this->loadModel('product')->getPriceByAttr($product, json_decode($goods->extra));

        $amount += $goods->price * $goods->count;

        $this->dao->insert(TABLE_ORDER_PRODUCT)->data($goods)->autoCheck()->exec();
    }

    /* Check valid products count. */
    $productCout = $this->dao->select("count(*) as count")->from(TABLE_ORDER_PRODUCT)->where('orderID')->eq($orderID)->fetch('count');
    if(!$productCout)  return array('result' => 'fail', 'message' => $this->lang->order->noProducts);

    $this->dao->update(TABLE_ORDER)->set('amount')->eq($amount)->where('id')->eq($orderID)->exec();
    $this->dao->delete()->from(TABLE_CART)->where('account')->eq($this->app->user->account)->andWhere('product')->in($this->post->product)->exec();
    if(!dao::isError()) return $orderID;
}

/**
 * Cancel an order.
 * 
 * @param  int    $orderID 
 * @access public
 * @return bool
 */
public function cancel($orderID)
{
    $this->dao->update(TABLE_ORDER)
        ->set('status')->eq('canceled')
        ->set('cancelReason')->eq($this->post->cancelReason)
        ->where('id')->eq($orderID)
        ->andWhere('account')->eq($this->app->user->account)
        ->exec();
    return !dao::isError();
}

/**
 * Process an order.
 * 
 * @param  object $order
 * @access public
 * @return bool
 */
public function processOrder($order)
{
    if($order->payStatus == 'paid') return true;

    $this->dao->update(TABLE_ORDER)
        ->set('sn')->eq($order->sn)
        ->set('payStatus')->eq('paid')
        ->set('paidDate')->eq(helper::now())
        ->where('id')->eq($order->id)->exec();

    if(dao::isError()) return false;

    $accounts  = $this->dao->select('mobile')->from(TABLE_USER)->where('account')->in($this->config->shop->sendto)->fetchAll();
    $mobiles   = array();
    $delimiter = isset($this->app->config->sms->delimiter) ? $this->app->config->sms->delimiter : ',';
    foreach($accounts as $account)
    {   
        if($account->mobile) $mobiles[] = $account->mobile;
    }

    /* Add service to the account. */
    $this->dao->update(TABLE_USER)->set('isCustomer')->eq(1)->where('account')->eq($order->account)->exec();
    $products = $this->dao->select('*')->from(TABLE_ORDER_PRODUCT)->where('orderID')->eq($order->id)->fetchAll();

    $service = new stdclass();
    if($this->config->request->defaultTime)
    {
        $days = $this->config->request->defaultTime * 365;
        $service->endTime = date('Y-m-d', strtotime("+{$days} day"));
    }
    else
    {
        $service->endTime = '0000-00-00';
    }

    foreach($products as $product)
    {
        $service->product = $product->productID;
        $this->loadModel('customer')->addService($order->accont, $service);
    }

    $content = sprintf($this->lang->order->toAdminContent, $this->config->site->name, $this->server->http_host, $order->account, $order->createdDate);

    $this->loadModel('mail')->send($this->config->shop->sendto, $this->lang->order->sendSubject, $content, true);

    $this->notice('pay', $order);

    return true;
}

/**
 * Delivery products of an order.
 * 
 * @param  int    $orderID 
 * @access public
 * @return void
 */
public function delivery($orderID)
{
    $order = $this->getByID($orderID);

    $delivery = fixer::input('post')
        ->add('deliveriedBy', $this->app->user->account)
        ->add('deliveryStatus', 'send')
        ->get();

    $this->dao->update(TABLE_ORDER)->data($delivery)->where('id')->eq($orderID)->exec();

    if(dao::isError()) return array('result' => 'fail', 'message' => dao::getError());

    if(isset($this->config->product->stock) and $this->config->product->stock)
    {
        $goodsList = $this->dao->select('*')->from(TABLE_ORDER_PRODUCT)->where('orderID')->eq($orderID)->fetchAll();
        foreach($goodsList as $goods)
        {
            $product = $this->dao->select('*')->from(TABLE_PRODUCT)->where('id')->eq($goods->productID)->fetch();
            if($product->amount < $goods->count)
            {
                return array('result' => 'fail', 'message' => strip_tags(sprintf($this->lang->order->lowStocks, $goods->productName)));
            }
        }
    }

    if($order->payment == 'COD' and isset($this->config->product->stock) and $this->config->product->stock) $this->fixStocks($orderID);

    $address     = json_decode($order->address);
    $expressList = $this->loadModel('tree')->getOptionMenu('express');

    $mobile  = $address->mobile;
    $content = sprintf($this->lang->order->toCustomerContent, $order->account, $this->config->site->name, $this->server->http_host, $expressList[$delivery->express], $delivery->waybill);
    $this->loadModel('sms')->send($mobile, $content);

    return array('result' => 'success', 'message' => $this->lang->saveSuccess, 'locate' => inlink('admin'));
}

/**
 * Print actions for order.
 * 
 * @param  int    $order 
 * @param  int    $btnLink 
 * @access public
 * @return void
 */
public function printActions($order, $btnLink = false)
{
    if(RUN_MODE == 'admin')
    { 
        if($btnLink) echo "<div class='btn-group'>";
        $class = $btnLink ? 'btn' : '';
        if(!$btnLink) echo html::a(inlink('view', "orderID=$order->id", "class='$class'"), $this->lang->order->view, "data-toggle='modal' class='$class'");

        if($order->status == 'normal')
        {
            /* Send link. */
            $disabled = ($order->deliveryStatus == 'not_send' and ($order->payment == 'COD' or ($order->payment != 'COD' and $order->payStatus == 'paid'))) ? '' : "disabled='disabled'"; 
            echo $disabled ? html::a('#', $this->lang->order->delivery, $disabled . "class='$class'") : html::a(helper::createLink('order', 'delivery', "orderID=$order->id"), $this->lang->order->delivery, "data-toggle='modal' class='$class'");

            /* Finish link. */
            $disabled = ($order->payStatus == 'paid' and $order->deliveryStatus == 'confirmed' and $order->status != 'finished' and $order->status != 'canceled') ? '' : "disabled='disabled'";
            echo $disabled ? html::a('#', $this->lang->order->finish, $disabled . "class='$class'") : html::a('javascript:;', $this->lang->order->finish, "data-rel='" . helper::createLink('order', 'finish', "orderID=$order->id") . "' class='finisher $class'");

            commonModel::printLink('order', 'editprice', "orderID={$order->id}", $this->lang->order->editPrice, "data-toggle='modal' class='$class'");

            $trackClass = $order->deliveryStatus == 'send' ? "class='$class'" : "class='disabled $class'";
            commonModel::printLink('order', 'track', "orderID={$order->id}", $this->lang->order->track, "{$trackClass} data-toggle='modal'");
        }

        /* message link. */
        $messageList  = $this->loadModel('message')->getByObject('message', 'order', $order->id);
        $waitMessages = $this->message->getWaitCountOfOrder($order->id);
        $disabled = empty($messageList) ? "disabled='disabled'" : '';
        echo $disabled ? html::a('#', $this->lang->order->message, $disabled . "class='$class'") : html::a(helper::createLink('order', 'message', "objectID=$order->id"), $this->lang->order->message . ($waitMessages ? '[' . $waitMessages . "]" : ''), "data-toggle='modal' class='$class'");

        if($btnLink) echo "</div>";

    }

    if(RUN_MODE == 'front' and $order->status == 'normal')
    {
        if($btnLink)
        {
            /* Pay link. */
            $disabled = ($order->payment != 'COD' and $order->payStatus != 'paid') ? '' : "disabled='disabled'";
            echo $disabled ? html::a('#', $this->lang->order->pay, "class='btn' $disabled") : html::a($this->createPayLink($order, $order->type), $this->lang->order->pay, "target='_blank' class='btn-go2pay btn warning'");

            /* Track link. */
            $disabled = ($order->deliveryStatus != 'not_send') ? '' : "disabled='disabled'";
            echo $disabled ? html::a('#', $this->lang->order->track, "class='btn' $disabled") : html::a(inlink('track', "orderID={$order->id}"), $this->lang->order->track, "data-rel='" . helper::createLink('order', 'confirmDelivery', "orderID=$order->id") . "' data-toggle='modal' class='btn btn-link'");

            /* Confirm link. */
            $disabled = ($order->deliveryStatus == 'send') ? '' : "disabled='disabled'";
            echo $disabled ? html::a('#', $this->lang->order->confirmReceived, "class='btn' $disabled") : html::a('javascript:;', $this->lang->order->confirmReceived, "data-rel='" . helper::createLink('order', 'confirmDelivery', "orderID=$order->id") . "' class='confirmDelivery btn primary'");

            /* Cancel link. */
            $disabled = ($order->status == 'normal' and $order->deliveryStatus == 'not_send' and $order->payStatus == 'not_paid') ? '' : "disabled='disabled'";
            echo $disabled ? html::a('#', $this->lang->order->cancel, "class='btn' $disabled") : html::a(helper::createLink('order', 'cancel', "orderID=$order->id"), $this->lang->order->cancel, "data-toggle='modal' class='btn btn-link'");

            /* message link. */
            echo html::a(helper::createLink('message', 'comment', "objectType=order&objectID=$order->id"), $this->lang->order->message, "data-toggle='modal' class='btn warning'");
        }
        else
        {
            /* Pay link. */
            $disabled = ($order->payment != 'COD' and $order->payStatus != 'paid') ? '' : "disabled='disabled'";
            echo $disabled ? html::a('#', $this->lang->order->pay, $disabled) : html::a($this->createPayLink($order, $order->type), $this->lang->order->pay, "target='_blank' class='btn-go2pay'");

            /* Track link. */
            $disabled = ($order->deliveryStatus != 'not_send') ? '' : "disabled='disabled'";
            echo $disabled ? html::a('#', $this->lang->order->track, $disabled) : html::a(inlink('track', "orderID={$order->id}"), $this->lang->order->track, "data-rel='" . helper::createLink('order', 'confirmDelivery', "orderID=$order->id") . "' data-toggle='modal'");

            /* Confirm link. */
            $disabled = ($order->deliveryStatus == 'send') ? '' : "disabled='disabled'";
            echo $disabled ? html::a('#', $this->lang->order->confirmReceived, $disabled) : html::a('javascript:;', $this->lang->order->confirmReceived, "data-rel='" . helper::createLink('order', 'confirmDelivery', "orderID=$order->id") . "' class='confirmDelivery'");

            /* Cancel link. */
            $disabled = ($order->status != 'finish') ? '' : "disabled='disabled'";
            echo $disabled ? html::a('#', $this->lang->order->cancel, $disabled) : html::a(helper::createLink('order', 'cancel', "orderID={$order->id}"), $this->lang->order->cancel, "data-toggle='modal'");

            /* message link. */
            echo html::a(helper::createLink('message', 'comment', "objectType=order&objectID=$order->id"), $this->lang->order->message, "data-toggle='modal'");
        }
    }
}

/**
 * Edit price of an order.
 * 
 * @param  int    $orderID 
 * @access public
 * @return bool
 */
public function editPrice($orderID)
{
    $this->dao->update(TABLE_ORDER)->set('amount')->eq($this->post->amount)->where('id')->eq($orderID)->exec();
    return !dao::isError();
}

/**
 * Get api by ID.
 * 
 * @param  int       $id 
 * @access public
 * @return object
 */
public function getApiByID($id)
{
    $api = $this->dao->select('*')->from(TABLE_ORDER_API)->where('id')->eq($id)->fetch();
    $api->params = json_decode($api->params);
    return $api;
}

/**
 * Get api list.
 * 
 * @param  string $orderBy 
 * @param  int    $pager 
 * @access public
 * @return array
 */
public function getApiList($orderBy = 'id_desc', $pager = null)
{
    $apiList = $this->dao->select('*')->from(TABLE_ORDER_API)->orderBy($orderBy)->page($pager)->fetchAll('id');
    foreach($apiList as $id => $api) $apiList[$id]->params = json_decode($api->params);
    return $apiList;
}

/**
 * Get api list by action.
 * 
 * @param  string    $action 
 * @access public
 * @return array
 */
public function getApiListByAction($action)
{
    $apiList = $this->dao->select('*')->from(TABLE_ORDER_API)->where('action')->eq($action)->fetchAll('id');
    foreach($apiList as $id => $api) $apiList[$id]->params = json_decode($api->params);
    return $apiList;
}

/**
 * Create api.
 * 
 * @access public
 * @return bool
 */
public function createApi()
{
    $api = fixer::input('post')->remove('createValue')->get();
    
    foreach($api->key as $i => $key)
    {
        if(empty($key)) continue;
        $api->params[$key] = $api->value[$i];
    }
    $api->params = helper::jsonEncode($api->params);

    $this->dao->insert(TABLE_ORDER_API)->data($api, 'key,value')->autoCheck()->exec();
    return !dao::isError();
}

/**
 * Update an api.
 * 
 * @param  int    $apiID 
 * @access public
 * @return bool
 */
public function updateApi($apiID)
{
    $data = fixer::input('post')->remove('createValue')->get();

    foreach($data->key as $i => $key)
    {
        if(empty($key)) continue;
        $data->params[$key] = $data->value[$i];
    }
    $data->params = helper::jsonEncode($data->params);

    $this->dao->update(TABLE_ORDER_API)->data($data, 'key,value')->where('id')->eq($apiID)->exec();
    return !dao::isError();
}

/**
 * Send notice action.
 * 
 * @param  int       $action 
 * @param  object    $order 
 * @access public
 * @return void
 */
public function notice($action, $order)
{
    $apiList = $this->getApiListByAction($action);

    foreach($apiList as $api)
    {
        $url = isset($api->url) ? $api->url : '';
        if(empty($url)) return false;

        $agent = $this->app->loadClass('snoopy');

        set_time_limit(60);
        if($api->method == 'get')
        {
            $params = '';
            if(!empty($api->params))
            {
                foreach($api->params as $key => $value)
                {
                    if(isset($this->lang->order->api->valueList[$value])) $value = $order->{$value};
                    $params .= $key . "=" . $value . "&";
                }
            }
            
            $linkSign = strpos($url, '?') !== false ? '&' : '?';
            $url      = $url . $linkSign . $params;
            $agent->fetch($url);
        }
        else
        {
            $params = array();
            if(!empty($api->params))
            {
                foreach($api->params as $key => $value)
                {
                    if(isset($this->lang->order->api->valueList[$value])) $value = $order->$value;
                    $params[$key] = $value;
                }
            }
            $agent->submit($url, $params); 
        }

        if(!empty($api->debug)) echo js::alert(json_encode($agent->results));
        return $agent->results;
    }
}
