<?php
/**
 * Create pay link. 
 * 
 * @param  object $order 
 * @param  string $type 
 * @access public
 * @return void
 */
public function createPayLink($order, $type = '') 
{   
    if($order->payment == 'alipay' or $order->payment == 'alipaySecured') return $this->createAlipayLink($order, $type);
    if($order->payment == 'wechatpay') return $this->createWechatPayLink($order, $type);
    return helper::createLink('order', 'check', "orderID=$order->id");
}

/**
 * Create wechatPay link.
 * 
 * @param  string $order
 * @param  string $type
 * @access public
 * @return void
 */
public function createWechatPayLink($order, $type = '')
{
    return inlink('wechatpay', "orderID=$order->id&type=$type");
}

/**
 * Get wechatpay config.
 * 
 * @param  string $type 
 * @access public
 * @return void
 */
public function getWechatpayConfig($type = '')
{
    $notifyURL = empty($type) ? inlink('processorder', "type=wechatpay&mode=notify") : helper::createLink($type, 'processorder', "type=wechat&mode=notify");

    $wechatpayConfig = array();
    $wechatpayConfig['appid']       = $this->config->wechatpay->appid;
    $wechatpayConfig['mch_id']      = $this->config->wechatpay->mch_id;
    $wechatpayConfig['apikey']      = $this->config->wechatpay->apikey;
    $wechatpayConfig['appsecret']   = $this->config->wechatpay->appsecret;
    $wechatpayConfig['notify_url']  = getWebRoot(true) . ltrim($notifyURL, '/');
    $wechatpayConfig['device_info'] = 'WEB';

    return $wechatpayConfig;
}

/**
 * Get order from wechatpay.
 * 
 * @param  string mode 
 * @access public
 * @return void
 */
public function getOrderFromWechatpay($mode)
{
    $this->app->loadClass('wechatpay', true);
    $wechatpay = new wechatpay($this->getWechatConfig());
    $data = $wechatpay->get_back_data();

    if($data['return_code'] == 'SUCCESS')
    {
        $orderID = $data['out_trade_no'];
        if($orderID) $orderID = $this->getRawOrder($orderID);
        $order = $this->getByID($orderID);                                                                                                           

        if(!$order) return false;

        $order->sn = $data['transaction_id'];
        return $order;
    }
    return false;
}

/**
 * Save wechatPay log. 
 * 
 * @access public
 * @return void
 */
public function saveWechatpayLog()
{   
    $content = date('Y-m-d H:i:s') . "\n";
    foreach($_POST as $key => $val) $content .= "$key = $val\n";
    $content .= "----------------\n";
    $logFile = $this->app->getTmpRoot() . 'log/wechatpay.log';
    $handle = fopen($logFile, 'a');
    fwrite($handle, $content);
    fclose($handle);
}

/**
 * Update status after pay.
 * 
 * @param  int    $orderID 
 * @param  int    $status 
 * @access public
 * @return void
 */
public function updatePayStatus($orderID, $status)
{
    $this->dao->update(TABLE_ORDER)->set('payStatus')->eq($status)->where('id')->eq($orderID)->exec();
}
