<?php
/** 
 * Get product list.
 * 
 * @param  array   $categories 
 * @param  string  $orderBy 
 * @param  int     $pager 
 * @param  bool    $image 
 * @param  array   $params 
 * @access public
 * @return array
 */
public function getList($categories, $orderBy, $pager = null, $image = false, $params = array()) 
{   
    $this->loadModel('file');
    $searchWord = $this->get->searchWord;
    $categoryID = $this->get->categoryID;

    $productIdList = array();
    if(!empty($params))
    {
        $isFirst = true; 
        foreach($params as $code => $value)
        {
            if($isFirst)
            {
                $productIdList = $this->dao->select('product')->from(TABLE_PRODUCT_ATTRIBUTE)
                    ->where('attribute')->eq($code)
                    ->andWhere('value')->like("%$value%")
                    ->fetchAll('product');
                if(empty($productIdList)) return array();
                $productIdList = join(',', array_keys($productIdList));
            }
            else
            {
                $productIdList = $this->dao->select('product')->from(TABLE_PRODUCT_ATTRIBUTE)
                    ->where('attribute')->eq($code)
                    ->andWhere('value')->like("%$value%")
                    ->andWhere('product')->in($productIdList)
                    ->fetchAll('product');
                if(empty($productIdList)) return array();
                $productIdList = join(',', array_keys($productIdList));
            }
            $isFirst = false;
        }
    }

    /* Get products(use groupBy to distinct products).  */
    $productIdList = $this->dao->select('id')->from(TABLE_RELATION)
        ->where('type')->eq('product')
        ->andWhere('category')->in($categories)
        ->beginIF(!empty($params))->andWhere('id')->in($productIdList)->fi()
        ->fetchPairs();

    if($image)
    {
        $objectIDList = $this->dao->setAutoLang(false)->select('`objectID`')->from(TABLE_FILE)
            ->where('objectType')->eq('product')
            ->andWhere('extension')->in($this->config->file->imageExtensions)
            ->beginIF(!empty($productIdList))->andWhere('objectID')->in($productIdList)->fi()
            ->orderBy('objectID desc') 
            ->fetchPairs();

        $productIdList = array_merge($productIdList, $objectIDList);
    }

    $products = $this->dao->select('*')->from(TABLE_PRODUCT)
        ->where('1 = 1')
        ->beginIF(!empty($categories) or $image)->andWhere('id')->in($productIdList)->fi()
        ->beginIF(RUN_MODE == 'front')->andWhere('status')->eq('normal')->fi()

        ->beginIF($searchWord)
        ->andWhere('name', true)->like("%{$searchWord}%")
        ->orWhere('brand')->like("%{$searchWord}%")
        ->orWhere('model')->like("%{$searchWord}%")
        ->orWhere('color')->like("%{$searchWord}%")
        ->orWhere('origin')->like("%{$searchWord}%")
        ->orWhere('keywords')->like("%{$searchWord}%")
        ->orWhere('`desc`')->like("%{$searchWord}%")
        ->orWhere('content')->like("%{$searchWord}%")
        ->markRight(1)
        ->fi()

        ->orderBy($orderBy)
        ->page($pager)
        ->fetchAll('id');
    if(!$products) return array();

    /* Get categories for these products. */
    $categories = $this->dao->select('t2.id, t2.name, t2.abbr, t2.alias, t2.unsaleable, t1.id AS product')
        ->from(TABLE_RELATION)->alias('t1')
        ->leftJoin(TABLE_CATEGORY)->alias('t2')->on('t1.category = t2.id')
        ->where('t2.type')->eq('product')
        ->andWhere('t1.id')->in(array_keys($products))
        ->fetchGroup('product', 'id');

    /* Get images for these products. */
    $images = $this->file->getByObject('product', array_keys($products), $isImage = true);

    foreach($products as $product)
    {
        /* Assign categories to it's product. */
        $product->categories = !empty($categories[$product->id]) ? $categories[$product->id] : array();
        $product->category   = current($product->categories);
        foreach($product->categories as $category)  $product->unsaleable = ($category->unsaleable and !$product->unsaleable);

        /* Assign images to it's product. */
        if(empty($images[$product->id])) continue;
        $product->image = new stdclass();
        if(isset($images[$product->id]))  $product->image->list = $images[$product->id];
        if(!empty($product->image->list)) $product->image->primary = $product->image->list[0];
        $product->desc = empty($product->desc) ? helper::substr(strip_tags($product->content), 250) : $product->desc;
    }

    return $products;
}



/**
 * Get prices for a product.
 * 
 * @param  int    $productID 
 * @access public
 * @return array
 */
public function getPrices($productID)
{
    return $this->dao->select('*')->from(TABLE_PRODUCT_PRICE)->where('product')->eq($productID)->fetchGroup('attribute', 'value');
}

/**
 * Set price for a product.
 * 
 * @param  object    $product 
 * @access public
 * @return void
 */
public function setPrice($product)
{
    $this->dao->update(TABLE_PRODUCT)
        ->set('price')->eq($this->post->price)
        ->set('promotion')->eq($this->post->promotion)
        ->where('id')->eq($product->id)
        ->autoCheck()
        ->exec();

    $this->dao->delete()->from(TABLE_PRODUCT_PRICE)->where('product')->eq($product->id)->exec();

    $prices = fixer::input('post')->remove('price')->remove('promotion')->get();

    $price = new stdclass();
    $price->product = $product->id;

    foreach($prices as $attribute => $setting)
    {
        $price->attribute = $attribute;
        foreach($setting as $value => $adjust)
        {
            $price->value = $value;
            $price->price = $adjust;
            $this->dao->insert(TABLE_PRODUCT_PRICE)->data($price)->autoCheck()->exec();
        }
    }
    return !dao::isError();
}

/**
 * Create a product.
 * 
 * @access public
 * @return int|bool
 */
public function create()
{
    $now = helper::now();
    $product = fixer::input('post')
        ->join('categories', ',')
        ->stripTags('content,desc', $this->config->allowedTags->admin)
        ->setDefault('price', 0)
        ->setDefault('amount', 0)
        ->setDefault('promotion', 0)
        ->setDefault('order', 0)
        ->add('author', $this->app->user->account)
        ->add('addedDate', $now)
        ->add('editedDate', $now)
        ->get();

    $product->alias    = seo::unify($product->alias, '-');
    $product->keywords = seo::unify($product->keywords, ',');

    $this->dao->insert(TABLE_PRODUCT)
        ->data($product, $skip = 'categories,uid,label,value,attribute')
        ->autoCheck()
        ->batchCheck($this->config->product->require->create, 'notempty')
        ->checkIF($product->mall, 'mall', 'URL')
        ->exec();

    $productID = $this->dao->lastInsertID();

    if(!$this->post->order) $this->dao->update(TABLE_PRODUCT)->set('order')->eq($productID)->where('id')->eq($productID)->exec();

    $attributes = $this->post->attribute;
    foreach($attributes as $code => $value)
    {
        $data = new stdclass();
        $data->product   = $productID;
        $data->attribute = $code;
        $data->value     = is_array($value) ? join('||', $value) : $value;
        $this->dao->insert(TABLE_PRODUCT_ATTRIBUTE)->data($data)->exec();
    }

    $attributes = $this->saveAttributes($productID);
    if($attributes === false) return false;

    $this->loadModel('file')->updateObjectID($this->post->uid, $productID, 'product');
    $this->file->copyFromContent($this->post->content, $productID, 'product');

    if(dao::isError()) return false;

    $this->loadModel('tag')->save($product->keywords);
    $this->processCategories($productID, $this->post->categories);

    $product = $this->getByID($productID);
    $product->attributes = $attributes; 
    $this->loadModel('search')->save('product', $product);

    return $productID;
}

/**
 * Update a product.
 * 
 * @param  int $productID 
 * @access public
 * @return void
 */
public function update($productID)
{
    $product = fixer::input('post')
        ->join('categories', ',')
        ->stripTags('content,desc', $this->config->allowedTags->admin)
        ->setDefault('price', 0)
        ->setDefault('amount', 0)
        ->setDefault('promotion', 0)
        ->setDefault('unsaleable', 0)
        ->add('editor', $this->app->user->account)
        ->add('editedDate', helper::now())
        ->get();

    $product->alias    = seo::unify($product->alias, '-');
    $product->keywords = seo::unify($product->keywords, ',');

    $this->dao->update(TABLE_PRODUCT)
        ->data($product, $skip = 'categories,uid,label,value,attribute')
        ->autoCheck()
        ->batchCheck($this->config->product->require->edit, 'notempty')
        ->checkIF($product->mall, 'mall', 'URL')
        ->where('id')->eq($productID)
        ->exec();

    $this->dao->delete()->from(TABLE_PRODUCT_ATTRIBUTE)->where('product')->eq($productID)->exec();
    $attributes = $this->post->attribute;
    foreach($attributes as $code => $value)
    {
        $data = new stdclass();
        $data->product   = $productID;
        $data->attribute = $code;
        $data->value     = is_array($value) ? join('||', $value) : $value;
        $this->dao->insert(TABLE_PRODUCT_ATTRIBUTE)->data($data)->exec();
    }

    $attributes = $this->saveAttributes($productID);
    if($attributes === false) return false;

    $this->loadModel('file')->updateObjectID($this->post->uid, $productID, 'product');
    $this->file->copyFromContent($this->post->content, $productID, 'product');

    if(dao::isError()) return false;

    $this->loadModel('tag')->save($product->keywords);
    $this->processCategories($productID, $this->post->categories);

    $product = $this->getByID($productID);
    if(empty($product)) return false;
    $product->attributes = $attributes; 
    return $this->loadModel('search')->save('product', $product);
}

/**
 * Get attributes of a product.
 * 
 * @param  int    $productID 
 * @access public
 * @return void
 */
public function getAttributes($productID)
{
    $attributes = $this->dao->select('*')->from(TABLE_PRODUCT_ATTRIBUTE)->where('product')->eq($productID)->fetchAll('attribute');
    foreach($attributes as $attribute)
    {
        if(strpos($attribute->value, '||') !== false) $attribute->value = explode('||', $attribute->value);
    }
    return $attributes;
}

/**
 * Print attributes for a product.
 * 
 * @param  object    $product 
 * @access public
 * @return string
 */
public function printAttributes($product)
{
    $attributes = $this->loadModel('attribute')->getPairs(array_keys($product->categories));
    $values     = $this->getAttributes($product->id);
    $prices     = $this->getPrices($product->id);
    $html       = '';

    if($this->app->device == 'mobile')
    {
        foreach($values as $code => $data)
        {
            $html .= "<tr><th>" . $attributes[$code] . "</th>";
            $html .= "<td>";
            $data->value = (array) $data->value;
            foreach($data->value as $item)
            {
                $base64 = helper::safe64Encode($item);
                if(isset($prices[$code][$base64]))
                {
                    $price = zget($prices[$code][$base64], 'price');
                    $html .= "<span class='price-label attr-selector' data-attribute='{$code}' data-value='{$item}' data-price='{$price}'>{$item}</span>";
                }
                else
                {
                    $html .= "<span class='price-label attr-selector' data-value='{$item}' data-attribute='{$code}'>{$item}</span>";
                }
                $html .= html::hidden("extra[$product->id][$code]", '', "class='{$code}-inputer'");
            }

            $html .= "</td></tr>";
        }
    }
    else
    {
        foreach($values as $code => $data)
        {
            $html .= "<li><span class='meta-name'>" . $attributes[$code] . "</span>";
            $html .= "<span class='meta-value'>";
            $data->value = (array) $data->value;
            foreach($data->value as $item)
            {
                $base64 = helper::safe64Encode($item);
                if(isset($prices[$code][$base64]))
                {
                    $price = zget($prices[$code][$base64], 'price');
                    $html .= "<span class='price-label attr-selector' data-attribute='{$code}' data-value='{$item}' data-price='{$price}'>{$item}</span>";
                }
                else
                {
                    $html .= "<span class='price-label attr-selector' data-value='{$item}' data-attribute='{$code}'>{$item}</span>";
                }
                $html .= html::hidden("extra[$product->id][$code]", '', "class='{$code}-inputer'");
            }

            $html .= "</span></li>";
        }
   
    }
    echo $html;
}

/**
 * Get price by attribute.
 * 
 * @param  object   $product 
 * @param  array    $extra 
 * @access public
 * @return void
 */
public function getPriceByAttr($product, $extra)
{
    $priceSetting = $this->loadModel('product')->getPrices($product->id);
    $price = $product->promotion > 0 ? $product->promotion : $product->price;

    if(!empty($extra))
    {
        foreach($extra as $attribute => $value)
        {
            $label      = helper::safe64Encode($value);
            $setting    = zget($priceSetting, $attribute, array());
            $extraPrice = zget($setting, $label, array());
            $price     += zget($extraPrice, 'price');
        }
    }
    return $price;
}
