<?php
/**
 * get search results of keywords.
 * 
 * @param  string    $keywords 
 * @param  object    $pager 
 * @access public
 * @return array
 */
public function getList($keywords, $pager)
{
    $spliter = $this->app->loadClass('spliter');
    $words   = explode(' ', seo::unify($keywords, ' '));

    $against = '';
    foreach($words as $word)
    {
        $splitedWords = $spliter->utf8Split($word);
        $against .= '"' . $splitedWords['words'] . '"'; 
    }
    
    $words = str_replace('"', '', $against);
    $words = str_pad($words, 5, '_');

    $scoreColumn = "((1 * (MATCH(title) AGAINST('{$against}' IN BOOLEAN MODE))) + (0.6 * (MATCH(title) AGAINST('{$against}' IN BOOLEAN MODE))) )";

    if($this->config->site->searchRange == 'mall')
    {
        $this->loadModel('product');
        $results = $this->dao->select("t1.*, {$scoreColumn} as score, t2.price, t2.promotion, t2.views, t2.unsaleable")
            ->from(TABLE_SEARCH_INDEX)->alias('t1')
            ->leftJoin(TABLE_PRODUCT)->alias('t2')->on('t1.objectID = t2.id')
            ->where("MATCH(t1.title,t1.content) AGAINST('+{$against}' IN BOOLEAN MODE) >= 1")
            ->andWhere('t1.status')->eq('normal')
            ->andWhere('t1.addedDate')->le(helper::now())
            ->beginIF(isset($this->config->site->searchRange) and $this->config->site->searchRange == 'mall')->andWhere('objectType')->eq('product')
            ->orderBy('score_desc, editedDate_desc')
            ->page($pager)
            ->fetchAll('id');
    }
    else
    {
        $results = $this->dao->select("*, {$scoreColumn} as score")
            ->from(TABLE_SEARCH_INDEX)
            ->where("MATCH(title,content) AGAINST('+{$against}' IN BOOLEAN MODE) >= 1")
            ->andWhere('status')->eq('normal')
            ->andWhere('addedDate')->le(helper::now())
            ->beginIF(isset($this->config->site->searchRange) and $this->config->site->searchRange == 'mall')->andWhere('objectType')->eq('product')
            ->orderBy('score_desc, editedDate_desc')
            ->page($pager)
            ->fetchAll('id');
    }

    $this->loadModel('file');
    $images = $this->dao->setAutoLang(false)->select('*')->from(TABLE_FILE)
        ->where('extension')->in($this->config->file->imageExtensions)
        ->andWhere('objectType')->in('article,product')
        ->orderBy('`primary`, editor_desc') 
        ->fetchGroup('objectID');

    foreach($images as $objectImages) $this->file->batchProcessFile($objectImages);

    foreach($results as $record)
    {
        $record->title   = str_replace('</span> ', '</span>', $this->decode($this->markKeywords($record->title, $words)));
        $record->title   = str_replace('_', '', $this->decode($this->markKeywords($record->title, $words)));
        $record->summary = $this->getSummary($record->content, $words);
        $record->summary = str_replace('_', '', $record->summary);

        if(empty($images[$record->objectID])) continue;
        $record->image = new stdclass();
        if(isset($images[$record->objectID]))  $record->image->list = $images[$record->objectID];
        if(!empty($record->image->list)) $record->image->primary = $record->image->list[0];
    }

    return $this->processLinks($results);
}

/**
 * Build all search index.
 * 
 * @access public
 * @return bool
 */
public function buildAllIndex($type, $lastID)
{
    $limit = 100;
    $categories = $this->dao->select('id,alias')->from(TABLE_CATEGORY)->fetchPairs();

    if($type == 'article')
    {
        $articles = $this->dao->select('t1.*, t2.category as category')
            ->from(TABLE_ARTICLE)->alias('t1')
            ->leftJoin(TABLE_RELATION)->alias('t2')->on("t1.id=t2.id")
            ->where('t2.type')->in('article,blog')
            ->beginIF($lastID)->andWhere('t1.id')->gt($lastID)
            ->orderBy('t1.id')
            ->limit($limit)
            ->fetchAll('id');

        if(empty($articles))
        {
            $type   = 'product';
            $lastID = 0;
        }
        else
        {
            foreach($articles as $article) 
            {
                $article->category = $categories[$article->category];
                $this->save($article->type, $article);
            }

            return array('type' => $type, 'count' => count($articles), 'lastID' => max(array_keys($articles)));
        }
    }

    if($type == 'product')
    {
        $products = $this->dao->select('t1.*, t2.category as category')
            ->from(TABLE_PRODUCT)->alias('t1')
            ->leftJoin(TABLE_RELATION)->alias('t2')->on("t1.id=t2.id")
            ->where('t2.type')->eq('product')
            ->beginIF($lastID)->andWhere('t1.id')->gt($lastID)
            ->limit($limit)
            ->fetchAll('id');

        if(empty($products))
        {
            $type   = 'page';
            $lastID = 0;
            if(isset($this->config->site->searchRange) and $this->config->site->searchRange == 'mall') return array('finished' => true);
        }
        else
        {
            foreach($products as $product)
            {
                $product->category = $categories[$product->category];
                $this->save('product', $product);
            }
            if(isset($this->config->site->searchRange) and $this->config->site->searchRange == 'mall') return array('finished' => true);
            return array('type' => $type, 'count' => count($products), 'lastID' => max(array_keys($products)));
        }
    }
    
    if($type == 'page')
    {
        $pages = $this->dao->select("*")
            ->from(TABLE_ARTICLE)
            ->where('type')->eq('page')
            ->beginIF($lastID)->andWhere('id')->gt($lastID)
            ->limit($limit)
            ->fetchAll('id');

        if(empty($pages))
        {
            $type   = 'thread';
            $lastID = 0;
        }
        else
        {
            foreach($pages as $page) $this->save('page', $page);
            return array('type' => $type, 'count' => count($pages), 'lastID' => max(array_keys($pages)));
        }
    }

    if($type == 'thread')
    {
        $threads = $this->dao->select("*, 'normal' as status")
            ->from(TABLE_THREAD)
            ->beginIF($lastID)->where('id')->gt($lastID)
            ->limit($limit)
            ->fetchAll('id');

        if(empty($threads))
        {
            $type   = 'book';
            $lastID = 0;
        }
        else
        {
            foreach($threads as $thread) $this->save('thread', $thread);
            return array('type' => $type, 'count' => count($threads), 'lastID' => max(array_keys($threads)));
        }
    }

    if($type == 'book')
    {
        $books    = $this->dao->select('id,alias')->from(TABLE_BOOK)->where('type')->eq('book')->fetchPairs();
        $articles = $this->dao->select('*')->from(TABLE_BOOK)->where('type')->eq('article')->fetchAll();

        foreach($articles as $article)
        {
            $pathes = explode(',', trim($article->path, ','));
            $bookID = $pathes[0];

            $article->book = $books[$bookID];
            $this->save('book', $article);
        }
        return array('finished' => true);
    }
}
